{*************************************************************}
{*                Unit Reports v1.0                          *}
{*    Copyright (c) 2000 by Dmitry Beloshistov [-=BDS=-]     *}
{*          Object for build text reports                    *}
{*************************************************************}
Unit Reports;

{--------------------------------------------------------------
 Report example:
 Input :  This is text from [   ] and [                    ] for records
 OutPut:  This is text from [111] and [ 0.123455           ] for records
---------------------------------------------------------------}

Interface
Uses Objects;

const
     ERR_NOERROR=0;
     ERR_NOFILE =1;
     ERR_IOERROR=2;
type

     PReport=^TReport;
     TReport=object(TObject)
               rmask,rout:Pstring;
               Fin,Fout:Text;
               ErrorCode:word;
               Constructor Init(repMask,repOutFile:String);
               Destructor Done;                                          virtual;
               Procedure BuildReportForm(Rec:Pointer);                   virtual;
               Function GetData(fldNum,fldLen:word; Rec:Pointer):string; virtual;
               Procedure BuildReportLine(S:String; Rec:Pointer);         virtual;
               Procedure BuildReport(DataSource:Pointer);                virtual;
               Procedure PrepareReport;                                  virtual;
               Procedure CloseReport;                                    virtual;
               Function DoMacro(S:String):String;                        virtual;
               Function Error:boolean;
             end;
             
Implementation
Uses FaFuncs,Dos,FaDate,FManage;
{--------------------------------------------}
Function TReport.DoMacro(S:String):String;
var Dt:DateTimeRec;
begin
 Now(Dt);
 if Pos('@DATE',S)>0 then S:=ReplaceStr(S,'@DATE',DateToStr(Dt));
 if Pos('@TIME',S)>0 then S:=ReplaceStr(S,'@TIME',TimeToStr(Dt));
 if Pos('@NOW',S)>0 then S:=ReplaceStr(S,'@NOW',DateTimeToStr(Dt));
 DoMacro:=S;
end;
{--------------------------------------------}
Constructor TReport.Init(repMask,repOutFile:String);
begin
 Inherited Init;
 ErrorCode:=ERR_NOERROR;
 if not FileExist(repMask) then ErrorCode:=ERR_NOFILE; {not found report file}
 rmask:=NewStr(repMask);
 rOut:=NewStr(repOutFile);
end;
{--------------------------------------------}
Destructor TReport.Done;
begin
 DisposeStr(Rmask);
 DisposeStr(rOut);
 Inherited Done;
end;
{--------------------------------------------}
Function TReport.Error:boolean;
begin
 Error:=(ErrorCode<>ERR_NOERROR);
end;
{--------------------------------------------}
Function TReport.GetData(fldNum,fldLen:Word; Rec:Pointer):string;
begin
 GetData:=''; {---- this is empty method!!! --------}
end;
{--------------------------------------------}
Procedure TReport.BuildReportLine(S:String; Rec:Pointer);
var CurFld:Word;
    CntSpace:Word;
    SBuf:String;
begin
 CurFld:=0;
 S:=DoMacro(S);
 if Pos('[',s)=0 then writeln(fout,s)
 else  begin
           while Pos('[',s)<>0 do
              begin
                CntSpace:=0;
                Sbuf:='';
                while (S<>'') and (s[1]<>'[') do
                  begin Sbuf:=SBuf+S[1]; delete(s,1,1); end;
                write(Fout,Sbuf);
                delete(s,1,1);
                while (s<>'') and (s[1]=' ') do
                 begin  delete(s,1,1); inc(CntSpace); end;
                Inc(CurFld);
                write(fout,GetData(CurFld,CntSpace,Rec));
                while (s[1]<>']') and (s<>'') do delete(s,1,1);
                delete(s,1,1);
             end;
         end;
  writeln(fout,s);
end;
{--------------------------------------------}
Procedure TReport.BuildReportForm(Rec:Pointer);
var s:string;
    CurFld:word;
    CntSpace:Word;
    SBuf:String;
begin
 Curfld:=0;
 while not eof(fin) do
  begin
   readln(fin,s);
   S:=DoMacro(S);
   if Pos('[',s)=0 then writeln(fout,s)
   else  begin
           while Pos('[',s)<>0 do
              begin
                CntSpace:=0;
                Sbuf:='';
                while (S<>'') and (s[1]<>'[') do
                  begin Sbuf:=SBuf+S[1]; delete(s,1,1); end;
                write(Fout,Sbuf);
                delete(s,1,1);
                while (s<>'') and (s[1]=' ') do
                 begin  delete(s,1,1); inc(CntSpace); end;
                Inc(CurFld);
                write(fout,GetData(CurFld,CntSpace,Rec));
                while (s[1]<>']') and (s<>'') do delete(s,1,1);
                delete(s,1,1);
             end;
           writeln(fout,s);
         end;
 end;
end;
{--------------------------------------------}
Procedure TReport.PrepareReport;
begin
{$I-}
 Assign(Fin,rmask^); reset(fin);
 Assign(fout,rOut^); rewrite(fout);
{$I+}
 if IOResult<>0 then ErrorCode:=ERR_IOERROR;
end;
{--------------------------------------------}
Procedure TReport.CloseReport;
begin
 {$I-}
 close(fin); flush(fout); close(fout);
 {$I+}
 if IOResult<>0 then ErrorCode:=ERR_IOERROR;
end;
{--------------------------------------------}
Procedure TReport.BuildReport(DataSource:Pointer);
begin
 PrepareReport;
 CloseReport;
end;
{--------------------------------------------}
{**********************************************************************}
{*                           Example                                  *}
{**********************************************************************}

(**************************************
type
    PFormRec=^TFormRec;             {user data record type}
    TFormRec=record
              Str:string;
              Number:integer;
             end;

    TMyReportForm=object(TReport)       {report as form}
                    Function GetData(fldNum,fldLen:Word;Rec:pointer):string; virtual;
                    Procedure BuildReport(DataSource:Pointer); virtual;
                  end;

    PTestDataSource=^TTestDataSource;   {array of records}
    TTestDataSource=array[0..1] of PFormRec;

    TMyReportLine=object(TMyReportForm) {report as table}
                    Procedure BuildReport(DataSource:Pointer); virtual;
                  end;

Function TMyReportForm.GetData(fldNum,fldLen:Word; Rec:Pointer):string;
var s:string;
begin
 case fldNum of
  1:s:=just(PFormRec(Rec)^.Str,fldLen,2);
  2:s:=just(IntToStr(PFormRec(Rec)^.Number),fldLen,2);
 end;
 GetData:='['+s+']';
end;

Procedure TMyReportForm.BuildReport(DataSource:Pointer);
begin
 PrepareReport;
 if not Error then BuildReportForm(DataSource);
 CloseReport;
end;

Procedure TMyReportLine.BuildReport(DataSource:Pointer);
var PTT:PTestDataSource;
    i:integer;
    S:String;
begin
 PrepareReport;
 if not Error then
  begin
   PTT:=DataSource; readln(Fin,S);
   for i:=0 to 1 do BuildReportLine(S,PTT^[i]);
  end;
 CloseReport;
end;

var RepF:TmyReportForm;
    RepL:TmyReportLine;
    PTT:PTestDataSource;
    Rec:PFormRec;
    i:integer;
begin
 New(PTT);
 for i:=0 to 1 do
  begin
   New(Rec);
   Rec^.Str:='String '+IntToStr(i); Rec^.Number:=i;
   PTT^[i]:=Rec;
  end;

 RepF.Init('fin.txt','routForm.txt');
 repF.buildreport(PTT^[0]);
 repF.done;

 RepL.Init('fin.txt','routLine.txt');
 repL.buildreport(PTT);
 repL.done;

 for i:=0 to 1 do Dispose(PFormRec(PTT^[i]));
 Dispose(PTT);
***************************************)
 
end.